@@ -7,7 +7,22 @@
     }
 }
 
-.generate_installation_order <- function(rang) {
+#' Create a Data Frame of The Resolved Result
+#' This function exports the results from [resolve()] to a data frame, which each row represents one installation step. The order of rows is the installation order. By installing packages in the specified order, one can install all the resolved packages without conflicts.
+#' @inheritParams export_rang
+#' @return A data frame ordered by installation order.
+#' @references
+#' Ripley, B. (2005) [Packages and their Management in R 2.1.0.](https://cran.r-project.org/doc/Rnews/Rnews_2005-1.pdf) R News, 5(1):8--11.
+#' @examples
+#' \donttest{
+#' if (interactive()) {
+#'     graph <- resolve(pkgs = c("openNLP", "LDAvis", "topicmodels", "quanteda"),
+#'                     snapshot_date = "2020-01-16")
+#'     generate_installation_order(graph)
+#' }
+#' }
+#' @export
+generate_installation_order <- function(rang) {
     dep <- fastmap::fastmap()
     version <- fastmap::fastmap()
     uid <- fastmap::fastmap()
@@ -68,7 +83,7 @@
         }
         loop_counter <- loop_counter + 1
         if (loop_counter > (length(needed_pkgrefs) * 5)) {
-            stop("Can't determine installation order. Please report the to the developers:\n", paste0(unfulfilled_pkgrefs, collapse = ","), call. = FALSE)
+            stop("Can't determine installation order. Please report this to the developers:\n", paste0(unfulfilled_pkgrefs, collapse = ","), call. = FALSE)
         }
     }
     ordered_pkgrefs <- c(installed_pkgrefs, noncranlike_pkgrefs)
@@ -110,189 +125,13 @@
                      as.character(check_cran_mirror), bioc_txt ,")"), con = con)
 }
 
-.query_mirror_validity <- function(mirror) {
-    if (mirror == "https://cran.r-project.org/") {
-        return(TRUE)
-    }
-    all_mirrors <- utils::getCRANmirrors()$URL
-    mirror %in% all_mirrors
-}
-
-.normalize_url <- function(mirror, https = TRUE) {
-    if (grepl("^http://", mirror)) {
-        mirror <- gsub("^http://", "https://", mirror)
-    }
-    if (!grepl("^https://", mirror)) {
-        mirror <- paste0("https://", mirror)
-    }
-    if (!grepl("/$", mirror)) {
-        mirror <- paste0(mirror, "/")
-    }
-    if (grepl("/+$", mirror)) {
-        mirror <- gsub("/+$", "/", mirror)
-    }
-    if (isTRUE(https)) {
-        return(mirror)
-    } else {
-        return(gsub("^https://", "http://", mirror))
-    }
-}
-
-.check_tarball_path <- function(tarball_path, x, dir = FALSE) {
-    ## raise error when tarball_path doesn't exist
-    if ((isFALSE(dir) && isFALSE(file.exists(tarball_path))) ||
-        (isTRUE(dir) && isFALSE(dir.exists(tarball_path)))) {
-        stop(x, " can't be cached.", call. = FALSE)
-    }
-    invisible()
-}
-
-.cache_pkg_cran <- function(x, version, cache_dir, cran_mirror, verbose) {
-    url <- paste(cran_mirror, "src/contrib/Archive/", x, "/", x, "_", version, ".tar.gz", sep = "")
-    tarball_path <- file.path(cache_dir, paste(x, "_", version, ".tar.gz", sep = ""))
-    tryCatch({
-        suppressWarnings(utils::download.file(url, destfile = tarball_path, quiet = !verbose))
-    }, error = function(e) {
-        ## is the current latest
-        url <- paste(cran_mirror, "src/contrib/", x, "_", version, ".tar.gz", sep = "")
-        utils::download.file(url, destfile = tarball_path, quiet = !verbose)
-    })
-    .check_tarball_path(tarball_path, x)
-}
-
-.cache_pkg_bioc <- function(x, version, cache_dir, bioc_mirror, bioc_version, verbose, uid) {
-    url <- paste(bioc_mirror, bioc_version, "/", uid, "/src/contrib/", x, "_", version, ".tar.gz", sep = "")
-    tarball_path <- file.path(cache_dir, paste(x, "_", version, ".tar.gz", sep = ""))
-    suppressWarnings(utils::download.file(url, destfile = tarball_path, quiet = !verbose))
-    .check_tarball_path(tarball_path, x)
-}
-
-.cache_pkg_github <- function(x, version, handle, source, uid, cache_dir, verbose) {
-    sha <- uid
-    tarball_path <- file.path(cache_dir, paste("raw_", x, "_", version, ".tar.gz", sep = ""))
-    utils::download.file(paste("https://api.github.com/repos/", handle, "/tarball/", sha, sep = ""), destfile = tarball_path,
-                         quiet = !verbose)
-    .check_tarball_path(tarball_path, x)
-}
-
-.cache_pkg_local <- function(x, version, cache_dir, uid) {
-    local_path <- uid
-    tarball_path <- file.path(cache_dir, paste("raw_", x, "_", version, ".tar.gz", sep = ""))
-    if (isTRUE(grepl("\\.tar.gz$|\\.tgz$", local_path))) {
-        ## it could be a valid source package, but don't trust it blindly, mark it as raw_
-        ## similar to github packages
-        file.copy(local_path, tarball_path)
-        return(.check_tarball_path(tarball_path, x))
-    }
-    if (.is_directory(local_path)) {
-        dir_pkg_path <- file.path(cache_dir, paste("dir_", x, "_", version, sep = ""))
-        res <- file.copy(from = local_path, to = cache_dir, recursive = TRUE, overwrite = TRUE)
-        res <- file.rename(from = file.path(cache_dir, x), to = dir_pkg_path)
-        return(.check_tarball_path(dir_pkg_path, x, dir = TRUE))
-    }
-}
-
-.cache_pkgs <- function(rang, output_dir, cran_mirror, bioc_mirror, verbose) {
-    installation_order <- .generate_installation_order(rang)
-    cache_dir <- file.path(output_dir, "cache")
-    if (!dir.exists(cache_dir)) {
-        dir.create(cache_dir)
-    }
-    for (i in seq(from = 1, to = nrow(installation_order), by = 1)) {
-        x <- installation_order$x[i]
-        source <- installation_order$source[i]
-        version <- installation_order$version[i]
-        handle <- installation_order$handle[i]
-        uid <- installation_order$uid[i]
-        if (source == "cran") {
-            .cache_pkg_cran(x = x, version = version, cache_dir = cache_dir,
-                        cran_mirror = cran_mirror, verbose = verbose)
-        }
-        if (source == "github") {
-            ## please note that these cached packages are not built
-            .cache_pkg_github(x = x, version = version, handle = handle,
-                          source = source, uid = uid,
-                          cache_dir = cache_dir, verbose = verbose)
-        }
-        if(source == "bioc") {
-            .cache_pkg_bioc(x = x, version = version, cache_dir = cache_dir,
-                            bioc_mirror = bioc_mirror, bioc_version = rang$bioc_version, verbose = verbose,
-                            uid = uid)
-        }
-        if(source == "local") {
-            ## please note that these cached packages are not built
-            .cache_pkg_local(x = x, version = version, cache_dir = cache_dir, uid = uid)
-        }
-
-    }
-    ## For #14, cache R source in the future here
-    invisible(output_dir)
-}
-
 .is_r_version_older_than <- function(rang, r_version = "1.3.1") {
     utils::compareVersion(rang$r_version, r_version) == -1
 }
 
-.insert_cache_dir <- function(dockerfile_content) {
-    rang_line <- which(dockerfile_content == "RUN Rscript rang.R")
-    c(dockerfile_content[1:(rang_line - 1)],
-      "COPY cache ./cache",
-      dockerfile_content[rang_line:length(dockerfile_content)])
-}
-
-.insert_materials_dir <- function(dockerfile_content) {
-    rang_line <- which(dockerfile_content == "COPY rang.R ./rang.R")
-    c(dockerfile_content[1:rang_line],
-      "COPY materials/ ./materials/",
-      dockerfile_content[(rang_line + 1):length(dockerfile_content)])
-}
-
-.generate_debian_eol_dockerfile_content<- function(r_version, lib, sysreqs_cmd, cache, debian_version = "lenny") {
-    dockerfile_content <- c(
-        paste0("FROM debian/eol:", debian_version),
-        "ENV TZ UTC",
-        "RUN ln -snf /usr/share/zoneinfo/$TZ /etc/localtime && echo $TZ > /etc/timezone && apt-get update -qq && apt-get install wget locales build-essential r-base-dev  -y",
-        "COPY rang.R ./rang.R",
-        "COPY compile_r.sh ./compile_r.sh",
-        paste("RUN", sysreqs_cmd),
-        paste("RUN bash compile_r.sh", r_version),
-        "CMD [\"R\"]")
-    if (!is.na(lib)) {
-        dockerfile_content[7] <- paste0("RUN mkdir ", lib, " && bash compile_r.sh ", r_version)
-    }
-    if (isTRUE(cache)) {
-        dockerfile_content <- c(dockerfile_content[1:5], "COPY cache ./cache", dockerfile_content[6:8])
-    }
-    return(dockerfile_content)
-}
-
-.generate_rocker_dockerfile_content <- function(r_version, lib, sysreqs_cmd, cache, image) {
-    dockerfile_content <- c("", "", "COPY rang.R ./rang.R", "RUN Rscript rang.R", "CMD [\"R\"]")
-    if (!is.na(lib)) {
-        dockerfile_content[4] <- paste0("RUN mkdir ", lib, " && Rscript rang.R")
-    }
-    dockerfile_content[1] <- paste0("FROM rocker/", image, ":", r_version)
-    dockerfile_content[2] <- paste("RUN", sysreqs_cmd)
-    if (image == "rstudio") {
-        dockerfile_content[5] <- "EXPOSE 8787"
-        dockerfile_content[6] <- "CMD [\"/init\"]"
-    }
-    if (isTRUE(cache)) {
-        dockerfile_content <- .insert_cache_dir(dockerfile_content)
-    }
-    return(dockerfile_content)
-}
-
-.generate_docker_readme <- function(output_dir,image) {
-    file.create(file.path(output_dir,"README"))
-    con <- file(file.path(output_dir,"README"), open="w")
-    readme <- readLines(system.file("readme_template.txt", package = "rang"))
-    readme <- gsub("__DATE__",Sys.Date(),readme)
-    readme <- gsub("__OUTPUT__",output_dir,readme)
-    readme <- gsub("__IMAGE__",image,readme)
-    writeLines(readme,file.path(output_dir,"README"))
-    close(con)
-    invisible(readme)
+## Wrap long line by breaking line at &&
+.generate_wrapped_line <- function(line) {
+    gsub("&&", "\\\\\n\t&&", line)
 }
 
 #' Export The Resolved Result As Installation Script
@@ -309,8 +148,9 @@
 #' @param check_cran_mirror logical, whether to check the CRAN mirror
 #' @param bioc_mirror character, which Bioconductor mirror to use
 #' @return `path`, invisibly
+#' @seealso [generate_installation_order()]
 #' @details The idea behind this is to determine the installation order of R packages locally. Then, the installation script can be deployed to another
-#' fresh R session to install R packages. [dockerize()] is a more reasonable way because a fresh R session with all system requirements
+#' fresh R session to install R packages. [dockerize()] and [apptainerize()] are more reasonable ways because a fresh R session with all system requirements
 #' is provided. The current approach does not work in R < 2.1.0.
 #' @export
 #' @references
@@ -342,7 +182,7 @@ export_rang <- function(rang, path, rang_as_comment = TRUE, verbose = TRUE, lib
     if (.is_r_version_older_than(rang, "3.3")) { #20
         cran_mirror <- .normalize_url(cran_mirror, https = FALSE)
     }
-    installation_order <- .generate_installation_order(rang)
+    installation_order <- generate_installation_order(rang)
     file.create(path)
     con <- file(path, open="w")
     if (.is_r_version_older_than(rang, "2.1")) {
@@ -395,7 +235,7 @@ export_renv <- function(rang, path = ".") {
         warning("Nothing to export.")
         return(invisible(NULL))
     }
-    pkg_df <- .generate_installation_order(rang)
+    pkg_df <- generate_installation_order(rang)
     pkg_list <- vector(mode = "list",length = nrow(pkg_df))
     names(pkg_list) <- pkg_df$x
     for(i in seq_len(nrow(pkg_df))){
@@ -410,7 +250,7 @@ export_renv <- function(rang, path = ".") {
             pkg_list[[i]][["RemoteType"]] <- "GitHub"
             pkg_list[[i]][["RemoteHost"]] <- "api.github.com"
             pkg_list[[i]][["RemoteRepo"]] <- pkg_df$x[i]
-            pkg_list[[i]][["RemoteUsername"]]<- strsplit(pkg_df$handle[i],"/")[[1]][1]
+            pkg_list[[i]][["RemoteUsername"]] <- strsplit(pkg_df$handle[i],"/")[[1]][1]
             pkg_list[[i]][["RemoteRef"]] = "HEAD"
             pkg_list[[i]][["RemoteSha"]] = pkg_df$uid[i]
         #   pkg_list[[i]][["Requirements"]] <- c()
@@ -436,55 +276,67 @@ export_renv <- function(rang, path = ".") {
     invisible(pkg_list)
 }
 
+.generate_container_readme <- function(output_dir, image, verb = c("dockerize", "apptainerize/singularize")) {
+    file.create(file.path(output_dir,"README"))
+    con <- file(file.path(output_dir,"README"), open="w")
+    verb <- match.arg(verb)
+    template_path <- switch(verb,
+                            "dockerize" = "docker_readme_template.txt",
+                            "apptainerize/singularize" = "apptainer_readme_template.txt")
+    readme <- readLines(system.file(template_path, package = "rang"))
+    readme <- gsub("__DATE__",Sys.Date(),readme)
+    readme <- gsub("__OUTPUT__",output_dir,readme)
+    readme <- gsub("__IMAGE__",image,readme)
+    writeLines(readme,file.path(output_dir,"README"))
+    close(con)
+    invisible(readme)
+}
 
-#' Dockerize The Resolved Result
-#'
-#' This function exports the result from [resolve()] to a Docker file. For R version >= 3.1.0, the Dockerfile is based on the versioned Rocker image.
-#' For R version < 3.1.0, the Dockerfile is based on Debian and it compiles R from source.
-#' @param output_dir character, where to put the Docker file and associated content
-#' @param materials_dir character, path to the directory containing additional resources (e.g. analysis scripts) to be copied into `output_dir` and in turn into the Docker container
-#' @param image character, which versioned Rocker image to use. Can only be "r-ver", "rstudio", "tidyverse", "verse", "geospatial"
-#' This applies only to R version >= 3.1
-#' @param cache logical, whether to cache the packages now. Please note that the system requirements are not cached. For query with non-CRAN packages, this option is strongly recommended. For query with local packages, this must be TRUE regardless of R version. For R version < 3.1, this must be also TRUE if there is any non-CRAN packages.
-#' @param no_rocker logical, whether to skip using Rocker images even when an appropriate version is available. Please keep this as `TRUE` unless you know what you are doing
-#' @param debian_version when Rocker images are not used, which EOL version of Debian to use. Can only be "lenny", "etch", "squeeze", "wheezy", "jessie", "stretch". Please keep this as default "lenny" unless you know what you are doing
-#' @param skip_r17 logical, whether to skip R 1.7.x. Currently, it is not possible to compile R 1.7.x (R 1.7.0 and R 1.7.1) with the method provided by  `rang`. It affects `snapshot_date` from 2003-04-16 to 2003-10-07. When `skip_r17` is TRUE and `snapshot_date` is within the aforementioned range, R 1.8.0 is used instead.
-#' @param ... arguments to be passed to `dockerize`
-#' @return `output_dir`, invisibly
-#' @inheritParams export_rang
-#' @inherit export_rang details
-#' @seealso [resolve()], [export_rang()]
-#' @references
-#' [The Rocker Project](https://rocker-project.org)
-#' Ripley, B. (2005) [Packages and their Management in R 2.1.0.](https://cran.r-project.org/doc/Rnews/Rnews_2005-1.pdf) R News, 5(1):8--11.
-#' @examples
-#' \donttest{
-#' if (interactive()) {
-#'     graph <- resolve(pkgs = c("openNLP", "LDAvis", "topicmodels", "quanteda"),
-#'                     snapshot_date = "2020-01-16")
-#'     dockerize(graph, ".")
-#' }
-#' }
-#' @export
-dockerize <- function(rang, output_dir, materials_dir = NULL, image = c("r-ver", "rstudio", "tidyverse", "verse", "geospatial"),
-                      rang_as_comment = TRUE, cache = FALSE, verbose = TRUE, lib = NA,
-                      cran_mirror = "https://cran.r-project.org/", check_cran_mirror = TRUE,
-                      bioc_mirror = "https://bioconductor.org/packages/",
-                      no_rocker = FALSE,
-                      debian_version = c("lenny", "squeeze", "wheezy", "jessie", "stretch"),
-                      skip_r17 = TRUE) {
+.insert_materials_dir <- function(containerfile_content, verb = c("dockerize", "apptainerize/singularize")) {
+    verb <- match.arg(verb)
+    if (verb == "dockerize") {
+        containerfile_content$COPY <- append(containerfile_content$COPY, "COPY materials/ ./materials/")
+        return(containerfile_content)
+    }
+    if (verb == "apptainerize/singularize") {
+        containerfile_content$FILES <- append(containerfile_content$FILES, "materials/ ./materials/")
+        return(containerfile_content)
+    }
+}
+
+.write_containerfile <- function(containerfile_content, path) {
+    content <- unlist(lapply(containerfile_content, .generate_wrapped_line))
+    writeLines(content, path)
+}
+
+## generate *ize() / *ise() content as a directory to output_dir
+## `containerfile_content` is a list from .generate_rocker_*_content() or .generate_debian_eol_*_content()
+.containerize <- function(rang, output_dir, materials_dir = NULL, post_installation_steps = NULL,
+                          image = c("r-ver", "rstudio", "tidyverse", "verse", "geospatial"),
+                          rang_as_comment = TRUE, cache = FALSE, verbose = TRUE, lib = NA,
+                          cran_mirror = "https://cran.r-project.org/", check_cran_mirror = TRUE,
+                          bioc_mirror = "https://bioconductor.org/packages/",
+                          no_rocker = FALSE,
+                          debian_version = c("lenny", "squeeze", "wheezy", "jessie", "stretch"),
+                          skip_r17 = TRUE,
+                          insert_readme = TRUE,
+                          copy_all = FALSE,
+                          verb = "dockerize", passive_verb = "dockerized",
+                          generate_rocker = .generate_rocker_dockerfile_content,
+                          generate_eol = .generate_debian_eol_dockerfile_content,
+                          output_file = "Dockerfile") {
     if (length(rang$ranglets) == 0) {
-        warning("Nothing to dockerize.")
+        warning(paste0("Nothing to ", verb), call. = FALSE)
         return(invisible(NULL))
     }
     if (missing(output_dir)) {
         stop("You must provide `output_dir`.", call. = FALSE)
     }
     if (!grepl("^ubuntu", rang$os)) {
-        stop("System dependencies of ", rang$os, " can't be dockerized.", call. = FALSE)
+        stop(paste0("System dependencies of ", rang$os, " can't be ", passive_verb), call. = FALSE)
     }
     if (.is_r_version_older_than(rang, "1.3.1")) {
-        stop("`dockerize` doesn't support this R version (yet):", rang$r_version, call. = FALSE)
+        stop(paste0("`", verb, "` doesn't support this R version (yet):", rang$r_version), call. = FALSE)
     }
     if (!is.null(materials_dir) && !(dir.exists(materials_dir))) {
         stop(paste0("The folder ", materials_dir, " does not exist"), call. = FALSE)
@@ -504,45 +356,195 @@ dockerize <- function(rang, output_dir, materials_dir = NULL, image = c("r-ver",
     if (!dir.exists(output_dir)) {
         dir.create(output_dir)
     }
-    rang_path <- file.path(output_dir, "rang.R")
-    export_rang(rang = rang, path = rang_path, rang_as_comment = rang_as_comment,
-                    verbose = verbose, lib = lib, cran_mirror = cran_mirror,
-                    check_cran_mirror = check_cran_mirror, bioc_mirror = bioc_mirror)
-    if (isTRUE(cache)) {
-        .cache_pkgs(rang, output_dir, cran_mirror, bioc_mirror, verbose)
+    if (dir.exists(file.path(output_dir, "inst/rang"))) {
+        base_dir <- file.path(output_dir, "inst/rang")
+        rel_dir <- "inst/rang"
+    } else {
+        base_dir <- output_dir
+        rel_dir <- ""
     }
+    if (rel_dir == "inst/rang" && isFALSE(copy_all)) {
+        .vcat(verbose, "`inst/rang` detected. `copy_all` is coerced to TRUE")
+        copy_all <- TRUE
+    }
+    rang_path <- file.path(base_dir, "rang.R")
+    export_rang(rang = rang, path = rang_path,
+                rang_as_comment = rang_as_comment,
+                verbose = verbose, lib = lib, cran_mirror = cran_mirror,
+                check_cran_mirror = check_cran_mirror, bioc_mirror = bioc_mirror)
     if (isTRUE(skip_r17) && rang$r_version %in% c("1.7.0", "1.7.1")) {
         r_version <- "1.8.0"
     } else {
         r_version <- rang$r_version
     }
+    if (isTRUE(cache)) {
+        .cache_pkgs(rang = rang, base_dir = base_dir, cran_mirror = cran_mirror,
+                    bioc_mirror = bioc_mirror, verbose = verbose)
+    }
     if (.is_r_version_older_than(rang, "3.1") || isTRUE(no_rocker)) {
-        file.copy(system.file("compile_r.sh", package = "rang"), file.path(output_dir, "compile_r.sh"),
+        file.copy(system.file("compile_r.sh", package = "rang"), file.path(base_dir, "compile_r.sh"),
                   overwrite = TRUE)
-        dockerfile_content <- .generate_debian_eol_dockerfile_content(r_version = r_version,
-                                                                      sysreqs_cmd = sysreqs_cmd, lib = lib,
-                                                                      cache = cache,
-                                                                      debian_version = debian_version)
+        containerfile_content <- generate_eol(r_version = r_version,
+                                              sysreqs_cmd = sysreqs_cmd, lib = lib,
+                                              cache = cache,
+                                              debian_version = debian_version,
+                                              post_installation_steps = post_installation_steps,
+                                              rel_dir = rel_dir,
+                                              copy_all = copy_all)
+        if (isTRUE(cache)) {
+            .cache_rsrc(r_version = r_version, base_dir = base_dir,
+                        verbose = verbose, cran_mirror = cran_mirror)
+            .cache_debian(debian_version = debian_version, base_dir = base_dir,
+                          verbose = verbose)
+        }
     } else {
-        dockerfile_content <- .generate_rocker_dockerfile_content(r_version = r_version,
-                                                                  sysreqs_cmd = sysreqs_cmd, lib = lib,
-                                                                  cache = cache, image = image)
+        containerfile_content <- generate_rocker(r_version = r_version,
+                                                 sysreqs_cmd = sysreqs_cmd, lib = lib,
+                                                 cache = cache, image = image,
+                                                 post_installation_steps = post_installation_steps,
+                                                 rel_dir = rel_dir,
+                                                 copy_all = copy_all)
     }
     if (!(is.null(materials_dir))) {
-        materials_subdir_in_output_dir <- file.path(output_dir, "materials")
+        materials_subdir_in_output_dir <- file.path(base_dir, "materials")
         if (isFALSE(dir.exists(materials_subdir_in_output_dir))) {
             dir.create(materials_subdir_in_output_dir)
         }
         file.copy(list.files(materials_dir, full.names = TRUE),
                   materials_subdir_in_output_dir,
                   recursive = TRUE)
-        dockerfile_content <- .insert_materials_dir(dockerfile_content)
+        containerfile_content <- .insert_materials_dir(containerfile_content, verb = verb)
+    }
+    ## This should be written in the root level, not base_dir
+    .write_containerfile(containerfile_content, file.path(output_dir, output_file))
+    if (isTRUE(insert_readme)) {
+        .generate_container_readme(output_dir = output_dir, image = image, verb = verb)
     }
-    writeLines(dockerfile_content, file.path(output_dir, "Dockerfile"))
-    .generate_docker_readme(output_dir = output_dir,image = image)
     invisible(output_dir)
 }
 
+#' Dockerize The Resolved Result
+#'
+#' This function exports the result from [resolve()] to a Docker file. For R version >= 3.1.0, the Dockerfile is based on the versioned Rocker image.
+#' For R version < 3.1.0, the Dockerfile is based on Debian and it compiles R from source.
+#' @param output_dir character, where to put the Docker file and associated content
+#' @param materials_dir character, path to the directory containing additional resources (e.g. analysis scripts) to be copied into `output_dir` and in turn into the Docker container
+#' @param post_installation_steps character, additional steps to be added before the `CMD` part of the Dockerfile, see an example below
+#' @param image character, which versioned Rocker image to use. Can only be "r-ver", "rstudio", "tidyverse", "verse", "geospatial"
+#' This applies only to R version >= 3.1
+#' @param cache logical, whether to cache the packages now. Please note that the system requirements are not cached. For query with non-CRAN packages, this option is strongly recommended. For query with local packages, this must be TRUE regardless of R version. For R version < 3.1, this must be also TRUE if there is any non-CRAN packages.
+#' @param no_rocker logical, whether to skip using Rocker images even when an appropriate version is available. Please keep this as `TRUE` unless you know what you are doing
+#' @param debian_version when Rocker images are not used, which EOL version of Debian to use. Can only be "lenny", "etch", "squeeze", "wheezy", "jessie", "stretch". Please keep this as default "lenny" unless you know what you are doing
+#' @param skip_r17 logical, whether to skip R 1.7.x. Currently, it is not possible to compile R 1.7.x (R 1.7.0 and R 1.7.1) with the method provided by  `rang`. It affects `snapshot_date` from 2003-04-16 to 2003-10-07. When `skip_r17` is TRUE and `snapshot_date` is within the aforementioned range, R 1.8.0 is used instead
+#' @param insert_readme logical, whether to insert a README file
+#' @param copy_all logical, whether to copy everything in the current directory into the container. If `inst/rang` is detected in `output_dir`, this is coerced to TRUE.
+#' @param ... arguments to be passed to `dockerize`
+#' @return `output_dir`, invisibly
+#' @inheritParams export_rang
+#' @inherit export_rang details
+#' @seealso [resolve()], [export_rang()], [use_rang()]
+#' @references
+#' [The Rocker Project](https://rocker-project.org)
+#' Ripley, B. (2005) [Packages and their Management in R 2.1.0.](https://cran.r-project.org/doc/Rnews/Rnews_2005-1.pdf) R News, 5(1):8--11.
+#' @examples
+#' \donttest{
+#' if (interactive()) {
+#'     graph <- resolve(pkgs = c("openNLP", "LDAvis", "topicmodels", "quanteda"),
+#'                     snapshot_date = "2020-01-16")
+#'     dockerize(graph, ".")
+#'     ## An example of using post_installation_steps to install quarto
+#'     install_quarto <- c("RUN apt-get install -y curl git && \\
+#'     curl -LO https://quarto.org/download/latest/quarto-linux-amd64.deb && \\
+#'     dpkg -i quarto-linux-amd64.deb && \\
+#'     quarto install tool tinytex")
+#'     dockerize(graph, ".", post_installation_steps = install_quarto)
+#' }
+#' }
+#' @export
+dockerize <- function(rang, output_dir, materials_dir = NULL, post_installation_steps = NULL,
+                      image = c("r-ver", "rstudio", "tidyverse", "verse", "geospatial"),
+                      rang_as_comment = TRUE, cache = FALSE, verbose = TRUE, lib = NA,
+                      cran_mirror = "https://cran.r-project.org/", check_cran_mirror = TRUE,
+                      bioc_mirror = "https://bioconductor.org/packages/",
+                      no_rocker = FALSE,
+                      debian_version = c("lenny", "squeeze", "wheezy", "jessie", "stretch"),
+                      skip_r17 = TRUE,
+                      insert_readme = TRUE,
+                      copy_all = FALSE) {
+    .containerize(rang = rang, output_dir = output_dir, materials_dir = materials_dir,
+                  post_installation_steps = post_installation_steps, image = image,
+                  rang_as_comment = rang_as_comment, cache = cache, verbose = verbose, lib = lib,
+                  cran_mirror = cran_mirror, check_cran_mirror = check_cran_mirror,
+                  bioc_mirror = bioc_mirror, no_rocker = no_rocker,
+                  debian_version = debian_version, skip_r17 = skip_r17, insert_readme = insert_readme,
+                  copy_all = copy_all, verb = "dockerize", passive_verb = "dockerized",
+                  generate_rocker = .generate_rocker_dockerfile_content,
+                  generate_eol = .generate_debian_eol_dockerfile_content,
+                  output_file = "Dockerfile")
+}
+
+#' Create an Apptainer/Singularity Definition File of The Resolved Result
+#'
+#' This function exports the result from [resolve()] to an Apptainer/Singularity definition file. For R version >= 3.1.0, the file is based on the versioned Rocker Docker image.
+#' For R version < 3.1.0, the Apptainer/Singularity definition is based on Debian and it compiles R from source.
+#' @param output_dir character, where to put the Apptainer/Singularity definition file and associated content
+#' @param materials_dir character, path to the directory containing additional resources (e.g. analysis scripts) to be copied into `output_dir` and in turn into the Apptainer/Singularity container
+#' @param post_installation_steps character, additional steps to be added before the in the end of `%post` section the Apptainer/Singularity definition file, see an example below
+#' @inheritParams dockerize
+#' @param ... arguments to be passed to `apptainerize`
+#' @return `output_dir`, invisibly
+#' @inheritParams export_rang
+#' @inherit export_rang details
+#' @seealso [resolve()], [export_rang()], [use_rang()]
+#' @references
+#' [Apptainer / Singularity](https://apptainer.org/)
+#'
+#' Kurtzer, G. M., Sochat, V., & Bauer, M. W. (2017) Singularity: Scientific containers for mobility of compute. PLOS ONE, 12(5):e0177459. \doi{10.1371/journal.pone.0177459}
+#'
+#' [The Rocker Project](https://rocker-project.org)
+#'
+#' Ripley, B. (2005) [Packages and their Management in R 2.1.0.](https://cran.r-project.org/doc/Rnews/Rnews_2005-1.pdf) R News, 5(1):8--11.
+#' @examples
+#' \donttest{
+#' if (interactive()) {
+#'     graph <- resolve(
+#'         pkgs = c("openNLP", "LDAvis", "topicmodels", "quanteda"),
+#'         snapshot_date = "2020-01-16"
+#'     )
+#'     apptainerize(graph, ".")
+#'     ## An example of using post_installation_steps to install quarto
+#'     install_quarto <- c("apt-get install -y curl git && \\
+#'     curl -LO https://quarto.org/download/latest/quarto-linux-amd64.deb && \\
+#'     dpkg -i quarto-linux-amd64.deb && \\
+#'     quarto install tool tinytex")
+#'     apptainerize(graph, ".", post_installation_steps = install_quarto)
+#' }
+#' }
+#' @export
+apptainerize <- function(rang, output_dir, materials_dir = NULL, post_installation_steps = NULL,
+                         image = c("r-ver", "rstudio", "tidyverse", "verse", "geospatial"),
+                         rang_as_comment = TRUE, cache = FALSE, verbose = TRUE, lib = NA,
+                         cran_mirror = "https://cran.r-project.org/", check_cran_mirror = TRUE,
+                         bioc_mirror = "https://bioconductor.org/packages/",
+                         no_rocker = FALSE,
+                         debian_version = c("lenny", "squeeze", "wheezy", "jessie", "stretch"),
+                         skip_r17 = TRUE,
+                         insert_readme = TRUE,
+                         copy_all = FALSE) {
+    .containerize(rang = rang, output_dir = output_dir, materials_dir = materials_dir,
+                  post_installation_steps = post_installation_steps, image = image,
+                  rang_as_comment = rang_as_comment, cache = cache, verbose = verbose, lib = lib,
+                  cran_mirror = cran_mirror, check_cran_mirror = check_cran_mirror,
+                  bioc_mirror = bioc_mirror, no_rocker = no_rocker,
+                  debian_version = debian_version, skip_r17 = skip_r17, insert_readme = insert_readme,
+                  copy_all = copy_all, verb = "apptainerize/singularize", passive_verb = "apptainerized/singularized",
+                  generate_rocker = .generate_rocker_apptainer_content,
+                  generate_eol = .generate_debian_eol_apptainer_content,
+                  output_file = "container.def")
+}
+
+## aliases
+
 #' @rdname dockerize
 #' @export
 dockerize_rang <- function(...) {
@@ -560,3 +562,46 @@ dockerise <- function(...) {
 dockerise_rang <- function(...) {
     dockerize(...)
 }
+
+#' @rdname apptainerize
+#' @export
+apptainerize_rang <- function(...) {
+    apptainerize(...)
+}
+
+#' @rdname apptainerize
+#' @export
+apptainerise <- function(...) {
+    apptainerize(...)
+}
+
+#' @rdname apptainerize
+#' @export
+apptainerise_rang <- function(...) {
+    apptainerize(...)
+}
+
+#' @rdname apptainerize
+#' @export
+singularize <- function(...) {
+    apptainerize(...)
+}
+
+
+#' @rdname apptainerize
+#' @export
+singularize_rang <- function(...) {
+    apptainerize(...)
+}
+
+#' @rdname apptainerize
+#' @export
+singularise <- function(...) {
+    apptainerize(...)
+}
+
+#' @rdname apptainerize
+#' @export
+singularise_rang <- function(...) {
+    apptainerize(...)
+}
