@@ -72,6 +72,42 @@ NULL
 
 .memo_search_bioc <- memoise::memoise(.bioc_package_history, cache = cachem::cache_mem(max_age = 60 * 60))
 
+.vcat <- function(verbose = TRUE, ...) {
+    if (isTRUE(verbose)) {
+        message(..., "\n")
+    }
+    invisible()
+}
+
+## data generation
+## ---
+## recipes <- list()
+## recipes["texlive"] <- "## install texlive\napt-get install -y pandoc pandoc-citeproc texlive"
+## recipes["texlivefull"] <- "## install texlive-full\napt-get install -y pandoc pandoc-citeproc texlive-full"
+## recipes["quarto"] <- "## install quarto (latest)\napt-get install -y curl git && curl -LO https://quarto.org/download/latest/quarto-linux-amd64.deb && dpkg -i quarto-linux-amd64.deb && quarto install tool tinytex && rm quarto-linux-amd64.deb"
+## recipes["clean"] <- "## Clean up caches\nrm -rf /var/lib/apt/lists/* && if [ -d \"$CACHE_PATH\" ]; then rm -rf $CACHE_PATH; fi"
+## recipes["make"] <- "## install GNU make\napt-get -y install make"
+## usethis::use_data(recipes, overwrite = TRUE)
+
+#' Recipes for Building Container Images
+#'
+#' A list containing several useful recipes for container building. Useful for the `post_installation_steps` argument of [dockerize()]. Available recipes are:
+#' * `texlive`: install pandoc and LaTeX, useful for rendering RMarkdown
+#' * `texlivefull`: Similar to the above, but install the full distribution of TeX Live (~ 3GB)
+#' * `quarto`: install quarto and tinytex
+#' * `clean`: clean up the container image by removing cache
+#' * `make`: install GNU make
+#' @examples
+#' \donttest{
+#' if (interactive()) {
+#'     graph <- resolve(pkgs = c("openNLP", "LDAvis", "topicmodels", "quanteda"),
+#'                     snapshot_date = "2020-01-16")
+#'     ## install texlive
+#'     dockerize(graph, ".", post_installation_steps = recipes[['texlive']])
+#' }
+#' }
+"recipes"
+
 ## internal data generation
 ## ---
 ### Supported OS Versions
@@ -82,7 +118,15 @@ NULL
 ## ### Bioconductor version history
 ## cached_biocver <- .biocver()
 ## attr(cached_biocver, "newest_date") <- max(cached_biocver$date)
-## usethis::use_data(supported_os, cached_rver, cached_biocver, internal = TRUE, overwrite = TRUE)
+## debian_version <- c("lenny", "squeeze", "wheezy", "jessie", "stretch")
+## .get_debian_urls <- function(debian_version, output_dir, verbose) {
+##     sha <- .gh(paste0("/repos/debuerreotype/docker-debian-eol-artifacts/branches/dist-",
+##                       debian_version))$commit$sha
+##     .gh(paste0("/repos/debuerreotype/docker-debian-eol-artifacts/contents/",
+##                debian_version, "/amd64/rootfs.tar.xz"), ref = sha)$download_url
+## }
+## debian_urls <- sapply(debian_version, .get_debian_urls)
+## usethis::use_data(supported_os, cached_rver, cached_biocver, debian_urls, internal = TRUE, overwrite = TRUE)
 
 ## test data upgrade
 ## ---
