@@ -2,6 +2,8 @@
     file.path(tempdir(), paste(sample(c(LETTERS, letters), 20, replace = TRUE), collapse = ""))
 }
 
+skip_if(isTRUE(getOption("SKIP_RESOLVE")))
+
 test_that("defensive programming", {
     expect_error(resolve("LDAvis", os = "windows"))
     expect_error(resolve("LDAvis", os = "opensuse-42.3"))
@@ -75,16 +77,16 @@ test_that("cache #17", {
     x <- readLines(file.path(temp_dir, "Dockerfile"))
     expect_false(any(grepl("^COPY cache", x)))
     expect_false(dir.exists(file.path(temp_dir, "cache")))
-    expect_false(file.exists(file.path(temp_dir, "cache", "LDAvis_0.3.2.tar.gz")))
-    expect_false(file.exists(file.path(temp_dir, "cache", "proxy_0.4-27.tar.gz")))
-    expect_false(file.exists(file.path(temp_dir, "cache", "RJSONIO_1.3-1.6.tar.gz")))
-    expect_silent(dockerize(rang_ok, output_dir = temp_dir, cache = TRUE, verbose = FALSE))
+    expect_false(file.exists(file.path(temp_dir, "cache/rpkgs", "LDAvis_0.3.2.tar.gz")))
+    expect_false(file.exists(file.path(temp_dir, "cache/rpkgs", "proxy_0.4-27.tar.gz")))
+    expect_false(file.exists(file.path(temp_dir, "cache/rpkgs", "RJSONIO_1.3-1.6.tar.gz")))
+    expect_silent(dockerize(rang_ok, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE))
     x <- readLines(file.path(temp_dir, "Dockerfile"))
     expect_true(any(grepl("^COPY cache", x)))
-    expect_true(dir.exists(file.path(temp_dir, "cache")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "LDAvis_0.3.2.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "proxy_0.4-27.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "RJSONIO_1.3-1.6.tar.gz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/rpkgs")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "LDAvis_0.3.2.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "proxy_0.4-27.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "RJSONIO_1.3-1.6.tar.gz")))
     ## expect_output(dockerize(rang_ok, output_dir = temp_dir, cache = TRUE, verbose = TRUE))
 })
 
@@ -94,16 +96,21 @@ test_that("cache for R < 3.1 and R >= 2.1", {
     rang_rio <- readRDS("../testdata/rang_rio_old.RDS")
     expect_equal(rang_rio$r_version, "3.0.1")
     temp_dir <- .generate_temp_dir()
-    dockerize(rang_rio, output_dir = temp_dir, cache = TRUE, verbose = FALSE)
+    dockerize(rang_rio, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE)
     x <- readLines(file.path(temp_dir, "Dockerfile"))
     expect_true(any(grepl("^COPY cache", x)))
-    expect_true(dir.exists(file.path(temp_dir, "cache")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "rio_0.1.1.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "stringr_0.6.2.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "digest_0.6.3.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "foreign_0.8-54.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "evaluate_0.4.7.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "testthat_0.7.1.tar.gz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/rpkgs")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "rio_0.1.1.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "stringr_0.6.2.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "digest_0.6.3.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "foreign_0.8-54.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "evaluate_0.4.7.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "testthat_0.7.1.tar.gz")))
+    ## Debian & rsrc
+    expect_true(dir.exists(file.path(temp_dir, "cache/rsrc")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rsrc", "R-3.0.1.tar.gz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/debian")))
+    expect_true(file.exists(file.path(temp_dir, "cache/debian", "rootfs.tar.xz")))
 })
 
 test_that("github correct querying; also #25", {
@@ -128,13 +135,13 @@ test_that("Non-cran must enforce caching ref #22", {
     temp_dir <- .generate_temp_dir()
     graph <- readRDS("../testdata/ancientsna.RDS")
     expect_equal(graph$ranglets[[1]]$pkgref, "github::cran/sna")
-    expect_error(dockerize(graph, output_dir = temp_dir, verbose = FALSE)) ## cache = FALSE
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA)
+    expect_error(dockerize(graph, output_dir = temp_dir, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE)) ## cache = FALSE
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
     temp_dir <- .generate_temp_dir()
     graph <- readRDS("../testdata/anciente1071.RDS")
     expect_equal(graph$ranglets[[1]]$pkgref, "cran::e1071")
-    expect_error(dockerize(graph, output_dir = temp_dir, verbose = FALSE), NA)
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA)
+    expect_error(dockerize(graph, output_dir = temp_dir, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
 })
 
 ## This should be tested. But this takes too long without cache.
@@ -171,13 +178,13 @@ test_that("cache bioc pkgs", {
     dockerize(rang_bioc, output_dir = temp_dir) ## cache = FALSE
     x <- readLines(file.path(temp_dir, "Dockerfile"))
     expect_false(any(grepl("^COPY cache", x)))
-    expect_false(dir.exists(file.path(temp_dir, "cache")))
-    expect_false(file.exists(file.path(temp_dir, "cache", "BiocGenerics_0.44.0.tar.gz")))
-    expect_silent(dockerize(rang_bioc, output_dir = temp_dir, cache = TRUE, verbose = FALSE))
+    expect_false(dir.exists(file.path(temp_dir, "cache/rpkgs")))
+    expect_false(file.exists(file.path(temp_dir, "cache/rpkgs", "BiocGenerics_0.44.0.tar.gz")))
+    expect_silent(dockerize(rang_bioc, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE))
     x <- readLines(file.path(temp_dir, "Dockerfile"))
     expect_true(any(grepl("^COPY cache", x)))
-    expect_true(dir.exists(file.path(temp_dir, "cache")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "BiocGenerics_0.44.0.tar.gz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/rpkgs")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "BiocGenerics_0.44.0.tar.gz")))
 })
 
 test_that("issue 68, correct querying of bioc packages from major releases", {
@@ -231,8 +238,8 @@ test_that("issue 89", {
     x <- resolve("bioc::GenomeInfoDbData", snapshot_date = "2023-01-01")
     expect_equal(x$ranglets[["bioc::GenomeInfoDbData"]]$original$x_uid, "data/annotation")
     temp_dir <- .generate_temp_dir()
-    expect_error(dockerize(x, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA)
-    expect_true(file.exists(file.path(temp_dir, "cache", "GenomeInfoDbData_1.2.9.tar.gz")))
+    expect_error(dockerize(x, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "GenomeInfoDbData_1.2.9.tar.gz")))
 })
 
 test_that("integration of renv to resolve", {
@@ -284,9 +291,9 @@ test_that("dockerize local package as tarball", {
     expect_error(suppressWarnings(graph <- resolve("local::../testdata/askpass_1.1.tar.gz", snapshot_date = "2023-01-01")), NA)
     expect_error(dockerize(graph, output_dir = temp_dir)) ## cache = FALSE
     temp_dir <- .generate_temp_dir()
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA) ## cache = FALSE
-    expect_true(file.exists(file.path(temp_dir, "cache", "sys_3.4.1.tar.gz")))
-    expect_true(file.exists(file.path(temp_dir, "cache", "raw_askpass_1.1.tar.gz")))
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "sys_3.4.1.tar.gz")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "raw_askpass_1.1.tar.gz")))
 })
 
 test_that("dockerize local package as tarball", {
@@ -296,9 +303,9 @@ test_that("dockerize local package as tarball", {
     expect_error(suppressWarnings(graph <- resolve("local::../testdata/askpass", snapshot_date = "2023-01-01")), NA)
     expect_error(dockerize(graph, output_dir = temp_dir)) ## cache = FALSE
     temp_dir <- .generate_temp_dir()
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA)
-    expect_true(file.exists(file.path(temp_dir, "cache", "sys_3.4.1.tar.gz")))
-    expect_true(dir.exists(file.path(temp_dir, "cache", "dir_askpass_1.1")))
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
+    expect_true(file.exists(file.path(temp_dir, "cache/rpkgs", "sys_3.4.1.tar.gz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/rpkgs", "dir_askpass_1.1")))
     x <- readLines(file.path(temp_dir, "rang.R"))
     expect_true(any(grepl("^## ## WARNING", x)))
 })
@@ -315,7 +322,7 @@ test_that("dockerize R1.3.1 local", {
     skip_on_cran()
     expect_error(suppressWarnings(graph <- resolve("../testdata/sna_0.3.tar.gz", snapshot_date = "2001-09-11")), NA)
     temp_dir <- .generate_temp_dir()
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA)
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
     x <- readLines(file.path(temp_dir, "rang.R"))
     expect_true(any(grepl("^## DEBUG INFO: CMD", x)))
 })
@@ -326,15 +333,40 @@ test_that("skip_r17", {
     expect_error(suppressWarnings(graph <- resolve("../testdata/sna_0.3.tar.gz", snapshot_date = "2003-04-17")), NA)
     expect_equal(graph$r_version, "1.7.0")
     temp_dir <- .generate_temp_dir()
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE), NA) ## skip_r17 = TRUE
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA) ## skip_r17 = TRUE
     x <- readLines(file.path(temp_dir, "Dockerfile"))
-    expect_true(any(grepl("^RUN bash compile_r\\.sh 1\\.8\\.0", x)))
-    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, skip_r17 = FALSE), NA)
+    expect_true(any(grepl("^RUN bash \\$COMPILE_PATH 1\\.8\\.0", x)))
+    expect_error(dockerize(graph, output_dir = temp_dir, cache = TRUE, verbose = FALSE, skip_r17 = FALSE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE), NA)
     x <- readLines(file.path(temp_dir, "Dockerfile"))
-    expect_false(any(grepl("^RUN bash compile_r\\.sh 1\\.8\\.0", x)))
-    expect_true(any(grepl("^RUN bash compile_r\\.sh 1\\.7\\.0", x)))
+    expect_false(any(grepl("^RUN bash \\$COMPILE_PATH 1\\.8\\.0", x)))
+    expect_true(any(grepl("^RUN bash \\$COMPILE_PATH 1\\.7\\.0", x)))
+    ## debian & rsrc
+    expect_true(dir.exists(file.path(temp_dir, "cache/rsrc")))
+    expect_true(file.exists(file.path(temp_dir, "cache/rsrc", "R-1.7.0.tgz")))
+    expect_true(dir.exists(file.path(temp_dir, "cache/debian")))
+    expect_true(file.exists(file.path(temp_dir, "cache/debian", "rootfs.tar.xz")))
 })
 
+test_that("dockerize with inst/rang", {
+    skip_if_offline()
+    skip_on_cran()
+    warns2 <- capture_warnings(rang_ok <- resolve(c("sna"), snapshot_date = "2001-10-01"))
+    temp_dir <- .generate_temp_dir()
+    dir.create(temp_dir)
+    use_rang(temp_dir, verbose = FALSE)
+    dockerize(rang_ok, output_dir = temp_dir, verbose = FALSE, cache = TRUE, cran_mirror = "https://cloud.r-project.org/", check_cran_mirror = FALSE)
+    expect_true("inst/rang/rang.R" %in% list.files(temp_dir, recursive = TRUE))
+    expect_false("rang.R" %in% list.files(temp_dir, recursive = TRUE))
+    expect_true(dir.exists(file.path(temp_dir, "inst/rang/cache")))
+    expect_true(dir.exists(file.path(temp_dir, "inst/rang/cache/rpkgs")))
+    expect_true(dir.exists(file.path(temp_dir, "inst/rang/cache/debian")))
+    expect_true(dir.exists(file.path(temp_dir, "inst/rang/cache/rsrc")))
+    expect_true("Dockerfile" %in% list.files(temp_dir, recursive = TRUE))
+    dockerfile <- readLines(file.path(temp_dir, "Dockerfile"))
+    expect_true("COPY . /" %in% dockerfile) ## coerced
+})
+
+
 ## always keep this at the very last
 
 test_that("issue 102 confusion between github and local pkgref", {
